/*
 * clock.c
 * Copyright (C) 1998 William H. Ball
 *
 * This file is part of TZones - A simple world clock for PalmOS
 *
 * TZones is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * TZones is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#define DO_NOT_ALLOW_ACCESS_TO_INTERNALS_OF_STRUCTS
#include <BuildDefines.h>
#ifdef DEBUG_BUILD
#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
#endif
#include <PalmOS.h>
#include <PalmCompatibility.h>
                                                                                                
#include "clock.h"
#include "graphics.h"
#include "teezee.h"

/* this function returns a approximation of 32767 * cos(x) where x is measured
 * in degrees. the algorithm is very fast and does not use division but has a large
 * error deviation (3%) although the algorithm is tuned such that cos(0) ans cos(90)
 * are exactly correct to reduce visual flaws.
 * 
 * clearly it would be more accurate to use a table which given the limited number
 * of input cases would not need to be that big. nethertheless  for this specific
 * application this implementation is sufficient.
 */
short IntCosine(short x)
{
	long sign;

	/* put x into the interval 0 <= x <= 90 */
	x = (x >   0 ? x : 0 - x);
	x = (x < 360 ? x : x % 360);
	x = (x < 180 ? x : 360 - x);
	x = (x <  90 ? (sign = 1, x) : (sign = -1, 180 - x));

	return (short) (sign * (32767l - ((16570l * x * x) >> 12)));
}

short IntSine(short x)
{
	return IntCosine(x - 90);
}

/* draw a clock, given a center x,y pos, radius of face, hour and minute 
 * this is really crude, but works OK for a 25-pixel diameter clock face
 */
void drawclock(short x, short y, short radius, short phour, short pminute, short reverse)
{
	const long hourScale = (3l * 32767l) / 2l; /* will give us an hour hand 2/3 radius */
	const long minuteScale = 32767l;
	
	short width, hangle, hx, hy, mx, my;
	char time[6];
	
	/* convert hour, minute and display under gap where the clock isn't (yet) */
	StrPrintF(time, "%02d:%02d", phour, pminute);
	width = FntCharsWidth(time, StrLen(time));
	WinDrawChars(time, StrLen(time), (x - (width / 2)), (y + radius + 7));

	/* calculate where the points of the hands go */
	hangle = (30*phour) + (pminute/2);
	hx = x + (((long) radius *   IntSine(hangle))      / hourScale);
	hy = y - (((long) radius * IntCosine(hangle))      / hourScale);
	mx = x + (((long) radius *   IntSine(6 * pminute)) / minuteScale);
	my = y - (((long) radius * IntCosine(6 * pminute)) / minuteScale);

	/* now check to see if user has selected reverse clocks */
	if (reverse == OFF) {
		/* draw clock border */
		circle(x, y, radius + 5, ON);
		circle(x, y, radius + 4, ON);
		circle(x, y, radius + 2, ON);

		/* hour hands */
		WinDrawLine(x, y, hx, hy);
		WinDrawLine(hx, hy, x + 1, y + 1);
		WinDrawLine(hx, hy, x - 1, y - 1);
		WinDrawLine(hx, hy, x + 1, y - 1);
		WinDrawLine(hx, hy, x - 1, y + 1);

		/* minute hands */
		WinDrawLine(x, y, mx, my);
		WinDrawLine(mx, my, x + 1, y + 1);
		WinDrawLine(mx, my, x - 1, y - 1);
		WinDrawLine(mx, my, x + 1, y - 1);
		WinDrawLine(mx, my, x - 1, y + 1);

		/* munge up the center of clock for effect */
		plot(x, y, OFF);
		plot(x - 1, y - 1, OFF);
		plot(x + 1, y + 1, OFF);
		plot(x - 1, y + 1, OFF);
		plot(x + 1, y - 1, OFF);
	}

	/* reverse clocks (a bit easier to read?) */
	if (reverse == ON) {
		/* draw black clock face w/border */
		fcircle(x, y, radius + 5, ON);
		circle(x, y, radius + 3, OFF);

		/* draw hour hand (erasing black face) */
		WinEraseLine(x, y, hx, hy);
		WinEraseLine(hx, hy, x + 1, y + 1);
		WinEraseLine(hx, hy, x - 1, y - 1);
		WinEraseLine(hx, hy, x + 1, y - 1);
		WinEraseLine(hx, hy, x - 1, y + 1);

		/* draw minute hand */
		WinEraseLine(x, y, mx, my);
		WinEraseLine(mx, my, x + 1, y + 1);
		WinEraseLine(mx, my, x - 1, y - 1);
		WinEraseLine(mx, my, x + 1, y - 1);
		WinEraseLine(mx, my, x - 1, y + 1);
	}
}
