/*
 * graphics.c
 * Copyright (C) 1987, 1998 William H. Ball
 *
 * This file is part of TZones - A simple world clock for PalmOS
 *
 * TZones is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * TZones is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#define DO_NOT_ALLOW_ACCESS_TO_INTERNALS_OF_STRUCTS
#include <BuildDefines.h>
#ifdef DEBUG_BUILD
#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
#endif
#include <PalmOS.h>
#include <PalmCompatibility.h>
                                                                                                
#include "graphics.h"
#include "teezee.h"

#define elementsof(x) (sizeof(x) / sizeof((x)[0]))

/* graphics primitives, declarations */

/* turn a pixel on/off @ x,y */
void plot(short x, short y, short mode)
{
	/* WinDrawPixel/WinErasePixel was not implemented until 3.5 */

	switch (mode) {
	case 0:
		WinEraseLine(x, y, x, y);
		break;
	case 1:
		WinDrawLine(x, y, x, y);
		break;
	default:
		break;
	}
}

/* draw a circle of radius around cx, cy, according to mode on/off */
void circle(short cx, short cy, short radius, short mode)
{
	short x, y, error;

	x = 0;
	y = radius;
	error = 3 - (2 * radius);
	while (x <= y) {
		plot(cx + x, cy + y, mode);
		plot(cx + x, cy - y, mode);
		plot(cx - x, cy + y, mode);
		plot(cx - x, cy - y, mode);
		plot(cx + y, cy + x, mode);
		plot(cx + y, cy - x, mode);
		plot(cx - y, cy + x, mode);
		plot(cx - y, cy - x, mode);
		if (error > 0) {
			error = error + 4 * (x - y) + 10;
			--y;
		} else {
			error = error + 4 * x + 6;
		}
		++x;
	}
}

/* draw a line from x1 to x2 along y, according to mode */
void hline(short x1, short x2, short y, short mode)
{
	switch (mode) {
	case 0:
		WinEraseLine(x1, y, x2, y);
		break;
	case 1:
		WinDrawLine(x1, y, x2, y);
		break;
	default:
		break;
	}
}

/* draw a filled circle of radius around x,y, according to mode */
void fcircle(short cx, short cy, short radius, short mode)
{
	short x, y, error;

	x = 0;
	y = radius;
	error = 3 - (2 * radius);

	while (x <= y) {
		hline(cx - x, cx + x, cy + y, mode);
		hline(cx - x, cx + x, cy - y, mode);
		hline(cx - y, cx + y, cy + x, mode);
		hline(cx - y, cx + y, cy - x, mode);
		if (error > 0) {
			error = error + 4 * (x - y) + 10;
			--y;
		} else {
			error = error + 4 * x + 6;
		}
		++x;
	}
}

/* resource pointers for earth graphics */
static BitmapPtr tzEarthBitmaps[31];

/* load bitmap resources */
void setEarth(void)
{
	Int16 resID[] = {
		bitmapID_earth00, bitmapID_earth01, bitmapID_earth02,
		    bitmapID_earth03,
		bitmapID_earth04, bitmapID_earth05, bitmapID_earth06,
		    bitmapID_earth07,
		bitmapID_earth08, bitmapID_earth09, bitmapID_earth10,
		    bitmapID_earth11,
		bitmapID_earth12, bitmapID_earth13, bitmapID_earth14,
		    bitmapID_earth15,
		bitmapID_earth16, bitmapID_earth17, bitmapID_earth18,
		    bitmapID_earth19,
		bitmapID_earth20, bitmapID_earth21, bitmapID_earth22,
		    bitmapID_earth23,
		bitmapID_earth24, bitmapID_earth25, bitmapID_earth26,
		    bitmapID_earth27,
		bitmapID_earth28, bitmapID_earth29, bitmapID_mercator
	};
	Int16 i;

	for (i = 0; i < elementsof(resID); i++) {
		/* this code initializes the tzEarthBitmaps array. it works on
		 * the assumption that the storage heap will not be altered
		 * during the execution of TZones (which is probably reasonable
		 * since TZones does not manipulate the storage heap). It doees
		 * however ignore almost every recommendation regarding PalmOS
		 * memory management!
		 */
		Handle resHandle =
		    (Handle) DmGetResource(bitmapRsc, resID[i]);
		tzEarthBitmaps[i] =
		    (BitmapPtr) MemHandleLock((VoidHand) resHandle);
		MemPtrUnlock(tzEarthBitmaps[i]);
		DmReleaseResource((VoidHand) resHandle);
	}

}

/* take the earth for a spin! */
void spinEarth(short num)
{
	const PointType earth = { 50, 25 };
	
	ErrNonFatalDisplayIf(num >= elementsof(tzEarthBitmaps), "array overflow");
	WinDrawBitmap(tzEarthBitmaps[num], earth.x, earth.y);
}

void eraseclocks(void)
{
	const RectangleType r = { { 0, 20 }, { 160, 140 } };

	WinEraseRectangle(&r, 0);
}
