////////////////////////////////////////////////////////////////////////
// clie.c
//
// Clie compatible substitutes for several Win...-calls
// 
// Arno Welzel / 2003-05-19
//
////////////////////////////////////////////////////////////////////////

#include <PalmOS.h>
#include <SonyCLIE.h>

UInt16 gHrLibApiVersion; // HRLib version
UInt16 gHrLibRefNum;     // HRLib reference number
Boolean gHrLibIsV2;      // True when HR Lib is ver.2 or later

/***********************************************************************
 *
 * FUNCTION:     GetHRLibVersion
 * 
 * DESCRIPTION:  Determine, if the Sony HR Lib is available
 *
 * PARAMETERS:   Pointer to variable to store lib reference handle
 *
 * RETURN:       0 if no HR lib is available, otherwise lib version
 *      
 ***********************************************************************/

UInt16 GetHRLibVersion(UInt16 *hrLibRefNumP)
{
	Err error = errNone;
	UInt16 refNum;
	UInt16 version;

	if((error = SysLibFind(sonySysLibNameHR, &refNum)) != errNone)
	{
		if(error == sysErrLibNotFound)
		{ /* couldn't find lib */
			error = SysLibLoad( 'libr', sonySysFileCHRLib, &refNum );
		}
	}

	if(!error )
	{
		// There's HR Library. Let's get API version
		*hrLibRefNumP = refNum;
		HRGetAPIVersion( refNum, &version ); // Assume this api success
	} else
	{
		// No HRLib
		version = 0;
	}

	return version;
}

/***********************************************************************
 *
 * FUNCTION:     clie_drawline
 * 
 * DESCRIPTION:  Clie compatible substitude for WinDrawLine
 *
 * PARAMETERS:   See WinDrawLine()
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/

void clie_drawline(Coord x1, Coord y1, Coord x2, Coord y2)
{
	if(gHrLibApiVersion) HRWinDrawLine(gHrLibRefNum, x1, y1, x2, y2);
	else WinDrawLine(x1, y1, x2, y2);
}

/***********************************************************************
 *
 * FUNCTION:     clie_invertline
 * 
 * DESCRIPTION:  Clie compatible substitude for WinInvertLine
 *
 * PARAMETERS:   See WinInvertLine()
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/

void clie_invertline(Coord x1, Coord y1, Coord x2, Coord y2)
{
	if(gHrLibApiVersion) HRWinInvertLine(gHrLibRefNum, x1, y1, x2, y2);
	else WinInvertLine(x1, y1, x2, y2);
}

/***********************************************************************
 *
 * FUNCTION:     clie_cliprectangle
 * 
 * DESCRIPTION:  Clie compatible substitude for WinClipRectangle
 *
 * PARAMETERS:   Coordinates of rectangle
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/

void clie_cliprectangle(RectangleType *rP)
{
	if(gHrLibApiVersion) HRWinClipRectangle(gHrLibRefNum, rP);
	else WinClipRectangle(rP);
}

/***********************************************************************
 *
 * FUNCTION:     clie_eraserectangle
 * 
 * DESCRIPTION:  Clie compatible substitude for WinEraseRectangle
 *
 * PARAMETERS:   see WinEraseRectangle()
 *
 * RETURN:       Nothing
 *      
 ***********************************************************************/

void clie_eraserectangle(RectangleType *rP, UInt16 cornerDiam)
{
	if(gHrLibApiVersion) HRWinEraseRectangle(gHrLibRefNum, rP, cornerDiam);
	else WinEraseRectangle(rP, cornerDiam);
}

/***********************************************************************
 *
 * FUNCTION:     clie_createoffscreenwindow
 * 
 * DESCRIPTION:  Clie compatible substitude for WinCreateOffscreenWindow
 *
 * PARAMETERS:   Width, hight, format, pointer for error return code
 *
 * RETURN:       Window handle
 *      
 ***********************************************************************/

WinHandle clie_createoffscreenwindow(Coord width, Coord height, WindowFormatType format, UInt16 *error)
{
	if(gHrLibApiVersion) return HRWinCreateOffscreenWindow(gHrLibRefNum, width, height, format, error);
	else return WinCreateOffscreenWindow(width, height, format, error);
}

/***********************************************************************
 *
 * FUNCTION:     clie_copyrectangle
 * 
 * DESCRIPTION:  Clie compatible substitude for WinCopyRectangle
 *
 * PARAMETERS:   see WinCopyRectangle()
 *
 * RETURN:       nothing
 *      
 ***********************************************************************/

void clie_copyrectangle(WinHandle srcWin, WinHandle dstWin, RectangleType *srcRect, Coord destX, Coord destY, WinDrawOperation mode)
{
	if(gHrLibApiVersion) HRWinCopyRectangle(gHrLibRefNum, srcWin, dstWin, srcRect, destX, destY, mode);
	else WinCopyRectangle(srcWin, dstWin, srcRect, destX, destY, mode);
}

/***********************************************************************
 *
 * FUNCTION:     clie_setclip
 * 
 * DESCRIPTION:  Clie compatible substitude for WinSetClip
 *
 * PARAMETERS:   see WinSetClip()
 *
 * RETURN:       nothing
 *      
 ***********************************************************************/

void clie_setclip(RectangleType *rP)
{
	if(gHrLibApiVersion) HRWinSetClip(gHrLibRefNum, rP);
	else WinSetClip(rP);
}

/***********************************************************************
 *
 * FUNCTION:     clie_getclip
 * 
 * DESCRIPTION:  Clie compatible substitude for WinGetClip
 *
 * PARAMETERS:   see WinGetClip()
 *
 * RETURN:       nothing
 *      
 ***********************************************************************/

void clie_getclip(RectangleType *rP)
{
	if(gHrLibApiVersion) HRWinGetClip(gHrLibRefNum, rP);
	else WinGetClip(rP);
}

/***********************************************************************
 *
 * FUNCTION:     clie_drawrectangleframe
 * 
 * DESCRIPTION:  Clie compatible substitude for WinDrawRectangleFrame
 *
 * PARAMETERS:   see WinDrawRectangleFrame()
 *
 * RETURN:       nothing
 *      
 ***********************************************************************/

void clie_drawrectangleframe(FrameType frame, const RectangleType *rP)
{
	if(gHrLibApiVersion) HRWinDrawRectangleFrame(gHrLibRefNum, frame, rP);
	else WinDrawRectangleFrame(frame, rP);
}

/***********************************************************************
 *
 * FUNCTION:     clie_getwindowextent
 * 
 * DESCRIPTION:  Clie compatible substitude for WinGetWindowExtent
 *
 * PARAMETERS:   see WinGetWindowExtent()
 *
 * RETURN:       nothing
 *      
 ***********************************************************************/

void clie_getwindowextent(Coord *extentX, Coord *extentY)
{
	if(gHrLibApiVersion) HRWinGetWindowExtent(gHrLibRefNum, extentX, extentY);
	else WinGetWindowExtent(extentX, extentY);
}

/***********************************************************************
 *
 * FUNCTION:     clie_drawbitmap
 * 
 * DESCRIPTION:  Clie compatible substitude for WinDrawBitmap
 *
 * PARAMETERS:   see WinDrawBitmap()
 *
 * RETURN:       nothing
 *      
 ***********************************************************************/

void clie_drawbitmap(BitmapPtr bitmapP, Coord x, Coord y)
{
	if(gHrLibApiVersion) HRWinDrawBitmap(gHrLibRefNum, bitmapP, x, y);
	else WinDrawBitmap(bitmapP, x, y);
}

/***********************************************************************
 *
 * FUNCTION:     clie_drawchars
 * 
 * DESCRIPTION:  Clie compatible substitude for WinDrawChars
 *
 * PARAMETERS:   see WinDrawChars()
 *
 * RETURN:       nothing
 *      
 ***********************************************************************/

void clie_drawchars(const Char *chars, Int16 len, Coord x, Coord y)
{
	if(gHrLibApiVersion) HRWinDrawChars(gHrLibRefNum, chars, len, x, y);
	else WinDrawChars(chars, len, x, y);
}
